package db.table;

import java.util.List;
import java.util.Optional;

import db.Table;
import db.annotation.DbTable;
import entity.Usager;

@DbTable(name = "Usager", entity = Usager.class)
public class UsagerTable extends Table<Usager> {
	
	private static UsagerTable instance;
	private static VilleTable vt = VilleTable.getInstance();
	private static EcoleTable et = EcoleTable.getInstance();
	
	private UsagerTable() {
		// Private constructor for singleton
	}
	
	public Usager getByCode(String code) {
		List<Usager> list = getByField("CODE", code);
		if(list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}
	
	public Optional<Usager> findByCodePostal(String code) {
		Usager obj = getByCode(code);
		if (obj == null) {
			return Optional.empty();
		}
		return Optional.of(obj);
	}

	@Override
	public Usager getById(long id) {
		return link(super.getById(id));
	}

	@Override
	public List<Usager> getByField(String fieldname, Object value) {
		return link(super.getByField(fieldname, value));
	}

	@Override
	public List<Usager> getWhere(List<String> where, List<Object> params) {
		return link(super.getWhere(where, params));
	}

	@Override
	public List<Usager> getAll() {
		return link(super.getAll());
	}

	@Override
	public void save(Usager obj) {
		vt.save(obj.ville);
		et.save(obj.ecole);
		obj.villeId = obj.ville.id;
		obj.ecoleId = obj.ecole.id;
		super.save(obj);
	}

	private Usager link(Usager obj) {
		if(obj == null) {
			return null;
		}
		obj.ville = vt.getById(obj.villeId);
		obj.ecole = et.getById(obj.ecoleId);
		return obj;
	}
	
	private List<Usager> link(List<Usager> list) {
		if(list.isEmpty()) {
			return list;
		}
		list.forEach(elt -> link(elt));
		return list;
	}
	
	public static UsagerTable getInstance() {
		if(instance == null) {
			instance = new UsagerTable();
		}
		return instance;
	}	

}
